@@ -42,11 +42,11 @@ x +- y + -
 
 from __future__ import division
 import re
-import csv
+import ast
 
 import numpy as N
 
-from ..compat import crange, cnext, CStringIO, citems
+from ..compat import crange, cnext, CStringIO
 from .. import utils
 from .. import document
 from .. import qtall as qt4
@@ -264,18 +264,20 @@ class DescriptorPart(object):
                     except ValueError:
                         dat = N.nan
                         self.errorcount += 1
-                        
+
                 elif self.datatype == 'string':
                     if string_re.match(val):
-                        # possible security issue:
-                        # regular expression checks this is safe
+                        conv = val
+                        if conv[0:1] != 'u':
+                            # a hack for python2
+                            conv = 'u' + val
                         try:
-                            dat = eval(val)
+                            dat = ast.literal_eval(conv)
                         except:
                             dat = val
                     else:
                         dat = val
-                        
+
                 elif self.datatype == 'date':
                     dat = utils.dateStringToDate(val)
 
@@ -468,7 +470,7 @@ class CSVStream(Stream):
                     # add on converted text - yuck - double conversion
                     self.remainingline.append('%e' % v)
                 else:
-                    self.remainingline.append(v)
+                    self.remainingline.append(t)
         return True
 
 class SimpleRead(object):
@@ -607,9 +609,9 @@ class SimpleRead(object):
         """Get a dict of the datasets read (main data part) and number
         of entries read."""
         out = {}
-        for name, data in citems(self.datasets):
+        for name in self.datasets:
             if name[-2:] == '\0D':
-                out[name[:-2]] = len(data)
+                out[name[:-2]] = len(self.datasets[name])
         return out
 
     def setOutput(self, out, linkedfile=None,
