@@ -118,7 +118,7 @@ class DatasetTableModel1D(qt4.QAbstractTableModel):
         if index.isValid():
             f = qt4.QAbstractTableModel.flags(self, index)
             ds = self.document.data.get(self.dsname)
-            if ds is not None and ds.editable():
+            if ds is not None and ds.editable:
                 f |= qt4.Qt.ItemIsEditable
             return f
         return qt4.Qt.ItemIsEnabled
@@ -277,7 +277,7 @@ class DatasetTableModelMulti(qt4.QAbstractTableModel):
             f = qt4.QAbstractTableModel.flags(self, index)
             dsname, colname, dsidx, colidx = self.colattrs[index.column()]
             ds = self.document.data.get(dsname)
-            if ds is not None and ds.editable():
+            if ds is not None and ds.editable:
                 f |= qt4.Qt.ItemIsEditable
             return f
         return qt4.Qt.ItemIsEnabled
@@ -389,13 +389,13 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
 
         if ds is not None and role == qt4.Qt.DisplayRole:
             v = self.xcent[section] if xaxis else self.ycent[section]
-            return '%i (%s)' % (section+1, setting.uilocale.toString(v))
+            return '%i (%s)' % (section+1, setting.ui_floattostring(v))
 
         elif ds is not None and role == qt4.Qt.ToolTipRole:
             v1 = self.xedge[section] if xaxis else self.yedge[section]
             v2 = self.xedge[section+1] if xaxis else self.yedge[section+1]
-            return u'%s\u2013%s' % (setting.uilocale.toString(v1),
-                                    setting.uilocale.toString(v2))
+            return u'%s\u2013%s' % (setting.ui_floattostring(v1),
+                                    setting.ui_floattostring(v2))
 
         return None
 
@@ -406,7 +406,7 @@ class DatasetTableModel2D(qt4.QAbstractTableModel):
         else:
             f = qt4.QAbstractTableModel.flags(self, index)
             ds = self.document.data.get(self.dsname)
-            if ds is not None and ds.editable():
+            if ds is not None and ds.editable:
                 f |= qt4.Qt.ItemIsEditable
             return f
 
@@ -451,7 +451,7 @@ class ViewDelegate(qt4.QStyledItemDelegate):
     def setEditorData(self, editor, index):
         """Override setData to use correct formatting."""
         if type(index.data()) is float:
-            txt = setting.uilocale.toString(index.data(), 'g', 8)
+            txt = setting.ui_floattostring(index.data())
             editor.setText(txt)
         else:
             qt4.QStyledItemDelegate.setEditorData(self, editor, index)
@@ -479,7 +479,7 @@ class DataEditDialog(VeuszDialog):
             (_('Insert row'), self.slotInsertRow),
             ):
             act = qt4.QAction(text, self)
-            self.connect(act, qt4.SIGNAL('triggered()'), slot)
+            act.triggered.connect(slot)
             self.datatableview.addAction(act)
         self.datatableview.setContextMenuPolicy( qt4.Qt.ActionsContextMenu )
 
@@ -496,12 +496,12 @@ class DataEditDialog(VeuszDialog):
 
         # select first item, if any or initialise if none
         if len(self.document.data) > 0:
-            self.selectDataset( sorted(self.document.data.keys())[0] )
+            self.selectDataset( sorted(self.document.data)[0] )
         else:
             self.slotDatasetsSelected([])
 
-        self.connect(self.dsbrowser.navtree, qt4.SIGNAL("selecteddatasets"),
-                     self.slotDatasetsSelected)
+        self.dsbrowser.navtree.selecteddatasets.connect(
+            self.slotDatasetsSelected)
 
         # connect buttons
         for btn, slot in ( (self.deletebutton, self.slotDatasetDelete),
@@ -511,7 +511,7 @@ class DataEditDialog(VeuszDialog):
                            (self.createbutton, self.slotDatasetCreate),
                            (self.editbutton, self.slotDatasetEdit),
                            ):
-            self.connect(btn, qt4.SIGNAL('clicked()'), slot)
+            btn.clicked.connect(slot)
 
         # menu for new button
         self.newmenu = qt4.QMenu()
@@ -519,7 +519,7 @@ class DataEditDialog(VeuszDialog):
                             (_('Text dataset'), self.slotNewTextDataset),
                             (_('Date/time dataset'), self.slotNewDateDataset) ):
             a = self.newmenu.addAction(text)
-            self.connect(a, qt4.SIGNAL('triggered()'), slot)
+            a.triggered.connect(slot)
         self.newbutton.setMenu(self.newmenu)
 
     def slotDatasetsSelected(self, names):
