@@ -31,7 +31,8 @@ import csv
 import time
 from collections import defaultdict
 
-from ..compat import citems, cstr, CStringIO, cbasestr, cpy3, cbytes, crepr
+from ..compat import citems, cstr, CStringIO, cbasestr, cpy3, cbytes, crepr, \
+    crange
 from .. import qtall as qt4
 import numpy as N
 
@@ -156,12 +157,6 @@ def pixmapAsHtml(pix):
     b64 = cbytes(buf.data().toBase64()).decode('ascii')
     return '<img src="data:image/png;base64,%s">' % b64
 
-def BoundCaller(function, *params):
-    """Wrap a function with its initial arguments."""
-    def wrapped(*args):
-        function( *(params+args) )
-    return wrapped
-
 def pythonise(text):
     """Turn an expression of the form 'A b c d' into 'A(b,c,d)'.
 
@@ -531,7 +526,7 @@ def round2delt(fin1, fin2):
     else:
         out1 = out2 = ''
 
-    for i in xrange(maxlog,-200,-1):
+    for i in crange(maxlog,-200,-1):
         p = 10**i
         d1, d2 = int(f1/p), int(f2/p)
         f1 -= int(d1)*p
@@ -582,3 +577,17 @@ def rrepr(val):
         return "[%s]" % ", ".join(l)
     else:
         return crepr(val)
+
+def escapeHDFDataName(name):
+    """Return escaped dataset name for saving in HDF5 files.
+    This is because names cannot include / characters in HDF5
+    """
+    name = name.replace('`', '`BT')
+    name = name.replace('/', '`SL')
+    return name.encode('utf-8')
+
+def unescapeHDFDataName(name):
+    """Return original name after being escaped."""
+    name = name.replace('`SL', '/')
+    name = name.replace('`BT', '`')
+    return name
